<?php
/**
 * File: Amazon AAWS
 * 	Amazon Product Lookup API
 *
 * Version:
 * 	2011.08.01
 *  by Pavel Kulbakin <p.kulbakin@gmail.com>
 *
 * Copyright:
 * 	2006-2009 LifeNexus Digital, Inc., and contributors.
 *
 * License:
 * 	Simplified BSD License - http://opensource.org/licenses/bsd-license.php
 *
 * See Also:
 * 	Tarzan - http://tarzan-aws.com
 * 	Amazon AAWS - http://aws.amazon.com/associates
 */


/*%******************************************************************************************%*/
// EXCEPTIONS

/**
 * Exception: AAWS_Exception
 * 	Default AAWS Exception.
 */
class AAWS_Exception extends Exception {}


/*%******************************************************************************************%*/
// MAIN CLASS

/**
 * Class: AmazonAAWS
 * 	Container for all Amazon Product Advertising API methods. Inherits additional methods from TarzanCore.
 *
 * Extends: TarzanCore
 *
 * Example Usage:
 * (start code)
 * require_once('tarzan.class.php');
 *
 * // Instantiate a new AmazonAAWS object using the settings from the config.inc.php file.
 * $aaws = new AmazonAAWS();
 *
 * // Instantiate a new AmazonAAWS object using these specific settings.
 * $aaws = new AmazonAAWS($key, $secret_key, $assoc_id);
 * (end)
 */
class AmazonAAWS extends TarzanCore
{
	public $api_version = '2011-08-01';
	
	/*%******************************************************************************************%*/
	// CONSTRUCTOR

	/**
	 * Method: __construct()
	 * 	The constructor
	 *
	 * Access:
	 * 	public
	 *
	 * Parameters:
	 * 	key - _string_ (Optional) Your Amazon API Key. If blank, it will look for the <AWS_KEY> constant.
	 * 	secret_key - _string_ (Optional) Your Amazon API Secret Key. If blank, it will look for the <AWS_SECRET_KEY> constant.
	 * 	assoc_id - _string_ (Optional) Your Amazon Associates ID. If blank, it will look for the <AWS_ASSOC_ID> constant.
	 *
	 * Returns:
	 * 	_boolean_ false if no valid values are set, otherwise true.
	 */
	public function __construct($key = null, $secret_key = null, $assoc_id = null)
	{
		if (!$key && !defined('AWS_KEY'))
		{
			throw new AAWS_Exception('No account key was passed into the constructor, nor was it set in the AWS_KEY constant.');
		}

		if (!$secret_key && !defined('AWS_SECRET_KEY'))
		{
			throw new AAWS_Exception('No account secret was passed into the constructor, nor was it set in the AWS_SECRET_KEY constant.');
		}

		if (!$assoc_id && !defined('AWS_ASSOC_ID'))
		{
			throw new AAWS_Exception('No Amazon Associates ID was passed into the constructor, nor was it set in the AWS_ASSOC_ID constant.');
		}

		return parent::__construct($key, $secret_key, null, $assoc_id);
	}


	/*%******************************************************************************************%*/
	// CORE FUNCTIONALITY

	/**
	 * Method: authenticate()
	 * 	Construct a URL to request from Amazon, request it, and return a formatted response.
	 *
	 * Access:
	 * 	public
	 *
	 * Parameters:
	 * 	action - _string_ (Required) Indicates the action to perform.
	 * 	opt - _array_ (Optional) Associative array of parameters. See the individual methods for allowed keys.
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Returns:
	 * 	<TarzanHTTPResponse> object
	 */
	public function authenticate($action, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		$hostname = AmazonAAWS_Locale::endpoint($locale);

		// Use alternate hostname, if one exists.
		if ($this->hostname)
		{
			$hostname = $this->hostname;
		}

		$return_curl_handle = false;
		$key_prepend = 'AWSAccessKeyId=' . $this->key . '&';

		// Manage the key-value pairs that are used in the query.
		$query['Operation'] = $action;
		$query['Service'] = 'AWSECommerceService';
		$query['SignatureMethod'] = 'HmacSHA256';
		$query['SignatureVersion'] = 2;
		$query['Timestamp'] = gmdate(DATE_AWS_ISO8601, time() + $this->adjust_offset);
		$query['Version'] = $this->api_version;
		
		// Merge in any options that were passed in
		if (is_array($opt))
		{
			$query = array_merge($query, $opt);
		}

		$return_curl_handle = isset($query['returnCurlHandle']) ? $query['returnCurlHandle'] : false;
		unset($query['returnCurlHandle']);

		// Do a case-insensitive, natural order sort on the array keys.
		uksort($query, 'strcasecmp');

		// Create the string that needs to be hashed.
		$canonical_query_string = $key_prepend . $this->util->to_signable_string($query);

		// Set the proper verb.
		$verb = HTTP_GET;

		// Set the proper path
		$path = '/onca/xml';

		// Prepare the string to sign
		$stringToSign = "$verb\n$hostname\n$path\n$canonical_query_string";

		// Hash the AWS secret key and generate a signature for the request.
		$query['Signature'] = $this->util->hex_to_base64(hash_hmac('sha256', $stringToSign, $this->secret_key));

		// Generate the querystring from $query
		$querystring = $key_prepend . $this->util->to_query_string($query);

		// Gather information to pass along to other classes.
		$helpers = array(
			'utilities' => $this->utilities_class,
			'request' => $this->request_class,
			'response' => $this->response_class,
		);

		// Compose the request.
		$request_url = $this->enable_ssl ? 'https://' : 'http://';
		$request_url .= $hostname;
		$request_url .= $path;
		$request_url .= '?' . $querystring;
		$request = new $this->request_class($request_url, $this->set_proxy, $helpers);

 		// If we have a "true" value for returnCurlHandle, do that instead of completing the request.
		if ($return_curl_handle)
 		{
 			return $request->prepRequest();
 		}

		// Send!
		$request->sendRequest();

 		// Prepare the response.
 		$headers = $request->getResponseHeader();
 		$headers['x-tarzan-requesturl'] = $request_url;
		$headers['x-tarzan-stringtosign'] = $stringToSign;
		
 		$data = new $this->response_class($headers, $request->getResponseBody(), $request->getResponseCode());

 		// Return!
		return $data;
	}


	/*%******************************************************************************************%*/
	// BROWSE NODE LOOKUP

	/**
	 * Method: browse_node_lookup()
	 * 	Given a browse node ID, <browse_node_lookup()> returns the specified browse node's name, children, and ancestors. The names and browse node IDs of the children and ancestor browse nodes are also returned. <browse_node_lookup()> enables you to traverse the browse node hierarchy to find a browse node.
	 * 	If you added your Associates ID to the config.inc.php file, or you passed it into the AmazonAAWS() constructor, it will be passed along in this request automatically.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	browse_node_id - _integer_ (Required) A positive integer assigned by Amazon that uniquely identifies a product category.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/BrowseNodeLookup.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 */
	public function browse_node_lookup($browse_node_id, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['BrowseNodeId'] = $browse_node_id;

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('BrowseNodeLookup', $opt, $locale);
	}


	/*%******************************************************************************************%*/
	// CART METHODS

	/**
	 * Method: cart_add()
	 * 	Enables you to add items to an existing remote shopping cart. <cart_add()> can only be used to place a new item in a shopping cart. It cannot be used to increase the quantity of an item already in the cart. If you would like to increase the quantity of an item that is already in the cart, you must use the <cart_modify()> operation.
	 * 	You add an item to a cart by specifying the item's OfferListingId, or ASIN and ListItemId. Once in a cart, an item can only be identified by its CartItemId. That is, an item in a cart cannot be accessed by its ASIN or OfferListingId. CartItemId is returned by <cart_create()>, <cart_get()>, and <cart_add()>.
	 * 	To add items to a cart, you must specify the cart using the CartId and HMAC values, which are returned by the <cart_create()> operation.
	 * 	If you added your Associates ID to the config.inc.php file, or you passed it into the AmazonAAWS() constructor, it will be passed along in this request automatically.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	cart_id - _string_ (Required) Alphanumeric token returned by <cart_create()> that identifies a cart.
	 * 	hmac - _string_ (Required) Encrypted alphanumeric token returned by <cart_create()> that authorizes access to a cart.
	 * 	offer_listing_id - _string|array_ (Required) Either a string containing the Offer ID to add, or an associative array where the Offer ID is the key and the quantity is the value. An offer listing ID is an alphanumeric token that uniquely identifies an item. Use the OfferListingId instead of an item's ASIN to add the item to the cart.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CartAdd.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 */
	public function cart_add($cart_id, $hmac, $offer_listing_id, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['CartId'] = $cart_id;
		$opt['HMAC'] = $hmac;

		if (is_array($offer_listing_id))
		{
			$count = 1;
			foreach ($offer_listing_id as $offer => $quantity)
			{
				$opt['Item.' . $count . '.OfferListingId'] = $offer;
				$opt['Item.' . $count . '.Quantity'] = $quantity;

				$count++;
			}
		}
		else
		{
			$opt['Item.1.OfferListingId'] = $offer_listing_id;
			$opt['Item.1.Quantity'] = 1;
		}

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('CartAdd', $opt, $locale);
	}

	/**
	 * Method: cart_clear()
	 * 	Enables you to remove all of the items in a remote shopping cart, including SavedForLater items. To remove only some of the items in a cart or to reduce the quantity of one or more items, use <cart_modify()>.
	 * 	To delete all of the items from a remote shopping cart, you must specify the cart using the CartId and HMAC values, which are returned by the <cart_create()> operation. A value similar to the HMAC, URLEncodedHMAC, is also returned. This value is the URL encoded version of the HMAC. This encoding is necessary because some characters, such as + and /, cannot be included in a URL. Rather than encoding the HMAC yourself, use the URLEncodedHMAC value for the HMAC parameter.
	 * 	<cart_clear()> does not work after the customer has used the PurchaseURL to either purchase the items or merge them with the items in their Amazon cart. Carts exist even though they have been emptied. The lifespan of a cart is 7 days since the last time it was acted upon. For example, if a cart created 6 days ago is modified, the cart lifespan is reset to 7 days.
	 * 	If you added your Associates ID to the config.inc.php file, or you passed it into the AmazonAAWS() constructor, it will be passed along in this request automatically.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	cart_id - _string_ (Required) Alphanumeric token returned by <cart_create()> that identifies a cart.
	 * 	hmac - _string_ (Required) Encrypted alphanumeric token returned by <cart_create()> that authorizes access to a cart.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CartClear.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 */
	public function cart_clear($cart_id, $hmac, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['CartId'] = $cart_id;
		$opt['HMAC'] = $hmac;

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('CartClear', $opt, $locale);
	}

	/**
	 * Method: cart_create()
	 * 	Enables you to create a remote shopping cart. A shopping cart is the metaphor used by most e-commerce solutions. It is a temporary data storage structure that resides on Amazon servers. The structure contains the items a customer wants to buy. In Amazon Associates Web Service, the shopping cart is considered remote because it is hosted by Amazon servers. In this way, the cart is remote to the vendor's web site where the customer views and selects the items they want to purchase.
	 * 	Once you add an item to a cart by specifying the item's ListItemId and ASIN, or OfferListing ID, the item is assigned a CartItemId and accessible only by that value. That is, in subsequent requests, an item in a cart cannot be accessed by its ListItemId and ASIN, or OfferListingId.
	 * 	Because the contents of a cart can change for different reasons, such as item availability, you should not keep a copy of a cart locally. Instead, use the other cart operations to modify the cart contents. For example, to retrieve contents of the cart, which are represented by CartItemIds, use <cart_get()>.
	 * 	Available products are added as cart items. Unavailable items, for example, items out of stock, discontinued, or future releases, are added as SaveForLaterItems. No error is generated. The Amazon database changes regularly. You may find a product with an offer listing ID but by the time the item is added to the cart the product is no longer available. The checkout page in the Order Pipeline clearly lists items that are available and those that are SaveForLaterItems.
	 * 	It is impossible to create an empty shopping cart. You have to add at least one item to a shopping cart using a single <cart_create()> request. You can add specific quantities (up to 999) of each item. <cart_create()> can be used only once in the life cycle of a cart. To modify the contents of the cart, use one of the other cart operations.
	 * 	Carts cannot be deleted. They expire automatically after being unused for 7 days. The lifespan of a cart restarts, however, every time a cart is modified. In this way, a cart can last for more than 7 days. If, for example, on day 6, the customer modifies a cart, the 7 day countdown starts over.
	 * 	If you added your Associates ID to the config.inc.php file, or you passed it into the AmazonAAWS() constructor, it will be passed along in this request automatically.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	offer_listing_id - _string|array_ (Required) Either a string containing the Offer ID to add, or an associative array where the Offer ID is the key and the quantity is the value. An offer listing ID is an alphanumeric token that uniquely identifies an item. Use the OfferListingId instead of an item's ASIN to add the item to the cart.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CartCreate.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 */
	public function cart_create($offer_listing_id, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();

		if (is_array($offer_listing_id))
		{
			$count = 1;
			foreach ($offer_listing_id as $offer => $quantity)
			{
				$opt['Item.' . $count . '.OfferListingId'] = $offer;
				$opt['Item.' . $count . '.Quantity'] = $quantity;

				$count++;
			}
		}
		else
		{
			$opt['Item.1.OfferListingId'] = $offer_listing_id;
			$opt['Item.1.Quantity'] = 1;
		}

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('CartCreate', $opt, $locale);
	}

	/**
	 * Method: cart_get()
	 * 	Enables you to retrieve the IDs, quantities, and prices of all of the items, including SavedForLater items in a remote shopping cart.
	 * 	Because the contents of a cart can change for different reasons, such as availability, you should not keep a copy of a cart locally. Instead, use <cart_get()> to retrieve the items in a remote shopping cart. To retrieve the items in a cart, you must specify the cart using the CartId and HMAC values, which are returned in the <cart_create()> operation. A value similar to HMAC, URLEncodedHMAC, is also returned.
	 * 	This value is the URL encoded version of the HMAC. This encoding is necessary because some characters, such as + and /, cannot be included in a URL. Rather than encoding the HMAC yourself, use the URLEncodedHMAC value for the HMAC parameter.
	 * 	<cart_get()> does not work after the customer has used the PurchaseURL to either purchase the items or merge them with the items in their Amazon cart.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	cart_id - _string_ (Required) Alphanumeric token returned by <cart_create()> that identifies a cart.
	 * 	hmac - _string_ (Required) Encrypted alphanumeric token returned by <cart_create()> that authorizes access to a cart.
	 * 	cart_item_id - _string_ (Required) Alphanumeric token that uniquely identifies an item in a cart. Once an item, specified by an ASIN or OfferListingId, has been added to a cart, you must use the CartItemId to refer to it. The other identifiers will not work.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CartGet.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 */
	public function cart_get($cart_id, $hmac, $cart_item_id, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['CartId'] = $cart_id;
		$opt['CartItemId'] = $cart_item_id;
		$opt['HMAC'] = $hmac;

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('CartGet', $opt, $locale);
	}

	/**
	 * Method: cart_modify()
	 * 	Enables you to change the quantity of items that are already in a remote shopping cart, move items from the active area of a cart to the SaveForLater area or the reverse, and change the MergeCart setting.
	 * 	To modify the number of items in a cart, you must specify the cart using the CartId and HMAC values that are returned in the <cart_create()> operation. A value similar to HMAC, URLEncodedHMAC, is also returned. This value is the URL encoded version of the HMAC. This encoding is necessary because some characters, such as + and /, cannot be included in a URL. Rather than encoding the HMAC yourself, use the URLEncodedHMAC value for the HMAC parameter.
	 * 	You can use <cart_modify()> to modify the number of items in a remote shopping cart by setting the value of the Quantity parameter appropriately. You can eliminate an item from a cart by setting the value of the Quantity parameter to zero. Or, you can double the number of a particular item in the cart by doubling its Quantity. You cannot, however, use <cart_modify()> to add new items to a cart.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	cart_id - _string_ (Required) Alphanumeric token returned by <cart_create()> that identifies a cart.
	 * 	hmac - _string_ (Required) Encrypted alphanumeric token returned by <cart_create()> that authorizes access to a cart.
	 * 	cart_item_id - _string|array_ (Required) Specifies an item to be modified in the cart where N is a positive integer between 1 and 10, inclusive. Up to ten items can be modified at a time.CartItemId is neither an ASIN nor an OfferListingId. It is, instead, an alphanumeric token returned by <cart_create()> and <cart_add()>. This parameter is used in conjunction with Item.N.Quantity to modify the number of items in a
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CartModify.html
	 * 
	 * Returns: <TarzanHTTPResponse> object
	 */
	public function cart_modify($cart_id, $hmac, $cart_item_id, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['CartId'] = $cart_id;
		$opt['HMAC'] = $hmac;

		if (is_array($cart_item_id))
		{
			$count = 1;
			foreach ($cart_item_id as $offer => $quantity)
			{
				$opt['Item.' . $count . '.CartItemId'] = $offer;
				$opt['Item.' . $count . '.Quantity'] = $quantity;

				$count++;
			}
		}
		else
		{
			$opt['Item.1.CartItemId'] = $offer_listing_id;
			$opt['Item.1.Quantity'] = 1;
		}

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('CartModify', $opt, $locale);
	}


	/*%******************************************************************************************%*/
	// ITEM METHODS

	/**
	 * Method: item_lookup()
	 * 	Given an Item identifier, the ItemLookup operation returns some or all of the item attributes, depending on the response group specified in the request. By default, <item_lookup()> returns an item’s ASIN, DetailPageURL, Manufacturer, ProductGroup, and Title of the item.
	 * 	If you added your Associates ID to the config.inc.php file, or you passed it into the AmazonAAWS() constructor, it will be passed along in this request automatically.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	item_id - _string_ (Required) A positive integer that unique identifies an item. The meaning of the number is specified by IdType. That is, if IdType is ASIN, the ItemId value is an ASIN. If ItemId is an ASIN, a search index cannot be specified in the request.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/ItemLookup.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 *
	 * Related: ItemSearch
	 */
	public function item_lookup($item_id, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['ItemId'] = $item_id;

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('ItemLookup', $opt, $locale);
	}

	/**
	 * Method: item_search()
	 * 	The <item_search()> operation returns items that satisfy the search criteria, including one or more search indices. <item_search()> is the operation that is used most often in requests. In general, when trying to find an item for sale, you use this operation.
	 * 	If you added your Associates ID to the config.inc.php file, or you passed it into the AmazonAAWS() constructor, it will be passed along in this request automatically.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	keywords - _string_ (Required) A word or phrase associated with an item. The word or phrase can be in various product fields, including product title, author, artist, description, manufacturer, and so forth. When, for example, the search index equals "MusicTracks", the Keywords parameter enables you to search by song title.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/ItemSearch.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 *
	 * Related: ItemLookup
	 */
	public function item_search($keywords, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['Keywords'] = $keywords;

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		if (!isset($opt['SearchIndex']) || empty($opt['SearchIndex']))
		{
			$opt['SearchIndex'] = 'All';
		}

		return $this->authenticate('ItemSearch', $opt, $locale);
	}

	/*%******************************************************************************************%*/
	// OTHER LOOKUP METHODS

	/**
	 * Method: similarity_lookup()
	 * 	Returns up to ten products per page that are similar to one or more items specified in the request. This operation is typically used to pique a customer's interest in buying something similar to what they've already ordered.
	 * 	If you specify more than one item, <similarity_lookup()> returns the intersection of similar items each item would return separately. Alternatively, you can use the SimilarityType parameter to return the union of items that are similar to any of the specified items. A maximum of ten similar items are returned; the operation does not return additional pages of similar items. If there are more than ten similar items, running the same request can result in different answers because the ten that are included in the response are picked randomly. The results are picked randomly only when you specify multiple items and the results include more than ten similar items.
	 * 	When you specify multiple items, it is possible for there to be no intersection of similar items. In this case, the operation returns an error.
	 * 	Similarity is a measurement of similar items purchased, that is, customers who bought X also bought Y and Z. It is not a measure, for example, of items viewed, that is, customers who viewed X also viewed Y and Z.
	 * 	If you added your Associates ID to the config.inc.php file, or you passed it into the AmazonAAWS() constructor, it will be passed along in this request automatically.
	 *
	 * Access: public
	 *
	 * Parameters:
	 * 	item_id - _string_ (Required) Specifies the item you want to look up. An ItemId is an alphanumeric identifier assigned to an item. You can specify up to ten ItemIds separated by commas.
	 * 	opt - _array_ (Optional) Associative array of parameters which can have the following keys:
	 * 	locale - _string_ (Optional) Which Amazon-supported locale do we use? Defaults to United States.
	 *
	 * Keys for the $opt parameter:
	 *  http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/CommonRequestParameters.html
	 * 	http://docs.amazonwebservices.com/AWSECommerceService/2011-08-01/DG/SimilarityLookup.html
	 *
	 * Returns: <TarzanHTTPResponse> object
	 *
	 * Related: ItemSearch using the Similarities response group 
	 */
	function similarity_lookup($item_id, $opt = null, $locale = AmazonAAWS_Locale::US)
	{
		if (!$opt) $opt = array();
		$opt['ItemId'] = $item_id;

		if (isset($this->assoc_id) && !empty($this->assoc_id))
		{
			$opt['AssociateTag'] = $this->assoc_id;
		}

		return $this->authenticate('SimilarityLookup', $opt, $locale);
	}

}
